
/**
 * Title:        Diff Word<p>
 * Description:  A diff that is not line based but word based.<p>
 * Copyright:    Copyright (c) Baptiste Lepilleur<p>
 * Company:      Gaia Creation<p>
 * @author Baptiste Lepilleur
 * @version 1.0
 */
package gaiacrtn.diffword;

import java.util.List;
import java.util.Vector;
import java.text.BreakIterator;
import gaiacrtn.util.TextReader;

/** Processes each line of a text to build a list of TextMarker that represents
 * the text.
 */
class TextMarkerLineProcessor implements TextReader.LineProcessor {
  private List _listMarker;
  private TextMarkerFactory _factoryMarker;
  protected int _nLineNo;

  /** Creates a line processor that build a list of word for the text.
   * @param factory Factory that is used to create the text marker.
   */
  public TextMarkerLineProcessor( TextMarkerFactory factory ) {
    _factoryMarker = factory;
    resetMarkers();
  }

  /** Empty the list of markers.
   * This method each should be called if more than one text is processed
   * with the same line processor, unless the result of the processing should
   * be appended to the previous result.
   */
  public void resetMarkers() {
    _listMarker = new Vector();
  }

  /** Gets the list of markers and empty the list.
   * @return List of markers that results from the processing.
   */
  public List takeMarkers() {
    List listMarker = _listMarker;
    resetMarkers();
    return listMarker;
  }

  /** Gets the list of markers.
   * @return List of markers that results from the processing.
   */
  public List markers() {
    return _listMarker;
  }

  /** Process a line of text.
   */
  public void processLine( String strLine, int nLineNo )
  {
    _nLineNo = nLineNo;
    BreakIterator breakIt = BreakIterator.getWordInstance();
    breakIt.setText( strLine );

    int start = breakIt.first();
    int end = breakIt.next();
    String strWord;

    while ( end != BreakIterator.DONE )
    {
      strWord = strLine.substring(start,end);
      processWord( strWord );

      start = end;
      end = breakIt.next();
    }

    processEndLine();
  }

  /** Processes a word of the line.
   *
   * A word is the sequence of letter that is between two BreakIteraor break.
   * In a typical case, a word may be a block of adjascent white space
   * characters, or a sequence of alphanumeric characters.
   * @param strWord Sequence of letter that was between the two last breaks
   *                of the BreakIterator.
   */
  protected void processWord( String strWord ) {
    addMarker( _factoryMarker.makeWord( strWord ) );
  }

  /** Processes the end of the line.
   */
  protected void processEndLine() {
    addMarker( _factoryMarker.makeEndLine() );
  }

  /** Adds a marker to the list markers.
   * @param marker Marker to add to the list.
   */
  protected void addMarker( TextMarker marker ) {
    _listMarker.add( marker );
  }
}
